@@ -1,3 +1,5 @@
+import numpy as np
+
 #   Plots elements names
 XS = "xs"
 YS = "ys"
@@ -5,7 +7,8 @@ VOLUME_NAMES = "volume_names"
 COMPO_NAMES = "compo_names"
 COLORS = "colors"
 EDGE_COLORS = "edge_colors"
-POLYGONS = "Polygons"
+POLYGONS = "polygons"
+GRID = "grid"
 
 #   Default field names
 MESH = "Mesh"
@@ -19,3 +22,6 @@ CSV = "CSV"
 X = (1., 0., 0.)
 Y = (0., 1., 0.)
 Z = (0., 0., 1.)
+
+# Cell name to define the outside world
+OUTSIDE = np.inf
