@@ -4,6 +4,7 @@ import panel as pn
 import os
 import functools
 
+from scivianna.data import Data2D
 from scivianna.interface.generic_interface import Geometry2D
 from scivianna.interface.option_element import OptionElement, BoolOption, FloatOption, IntOption, SelectOption, StringOption
 from scivianna.components.overlay_component import Overlay
@@ -11,10 +12,11 @@ from scivianna.components.server_file_browser import ServerFileBrowser
 from scivianna.enums import GeometryType, UpdateEvent, VisualizationMode
 from scivianna.slave import ComputeSlave
 
-from scivianna.utils.polygonize_tools import PolygonElement, PolygonSorter
-from scivianna.plotter_2d.bokeh import Bokeh2DPlotter
+from scivianna.utils.polygon_sorter import PolygonSorter
+from scivianna.plotter_2d.polygon.bokeh import Bokeh2DPolygonPlotter
+from scivianna.plotter_2d.grid.bokeh import Bokeh2DGridPlotter
 from scivianna.plotter_2d.generic_plotter import Plotter2D
-from scivianna.constants import GEOMETRY, COLORS, COMPO_NAMES, VOLUME_NAMES, POLYGONS
+from scivianna.constants import GEOMETRY
 from scivianna.utils.color_tools import beautiful_color_maps
 
 profile_time = bool(os.environ["VIZ_PROFILE"]) if "VIZ_PROFILE" in os.environ else 0
@@ -26,6 +28,7 @@ pn.extension(
     "tabulator", design="material", template="material", loading_indicator=True
 )  # necessary to have Panel/Bokeh in a Notebook
 
+
 class VisualizationPanel:
     """Visualisation panel associated to a code."""
 
@@ -51,14 +54,21 @@ class VisualizationPanel:
     """ Panel name
     """
 
-    current_polygons: Dict[str, Any]
-    """ Displayed polygons and their properties.
+    current_data: Dict[str, Any]
+    """ Displayed data and their properties.
     """
-    update_event:Union[UpdateEvent, List[UpdateEvent]] = UpdateEvent.RECOMPUTE
+    update_event: Union[UpdateEvent, List[UpdateEvent]] = UpdateEvent.RECOMPUTE
+    """ On what event does the panel recompute itself
+    """
+    sync_field: bool = False
     """ On what event does the panel recompute itself
     """
 
-    def __init__(self, slave: ComputeSlave, name=""):
+    display_polygons: bool
+    """ Display as polygons or as a 2D grid.
+    """
+
+    def __init__(self, slave: ComputeSlave, name="", display_polygons: bool = True):
         """Visualization panel constructor
 
         Parameters
@@ -67,19 +77,28 @@ class VisualizationPanel:
             ComputeSlave object to which request the plots.
         name : str
             Name of the panel.
+        display_polygons : bool
+            Display as polygons or as a 2D grid.
         """
         self.name = name
         self.copy_index = 0
         self.slave = slave
         self.bounds_row = None
         self.update_polygons = False
-        """Need to update the polygons at the next async call"""
+        """Need to update the data at the next async call"""
+        self.display_polygons = display_polygons
 
-        code_interface:Geometry2D = self.slave.code_interface
-        self.geometry_type:GeometryType = code_interface.geometry_type
-        self.rasterized:bool = code_interface.rasterized
+        code_interface: Geometry2D = self.slave.code_interface
+        self.polygon_sorter = PolygonSorter()
 
-        self.polygon_worker = PolygonSorter()
+        self.field_change_callback: Callable = None
+        """Function to call when the field is changed"""
+
+        assert issubclass(code_interface, Geometry2D), \
+            f"A VisualizationPanel can only be given a Geometry2D interface slave, received {code_interface}."
+
+        self.geometry_type: GeometryType = code_interface.geometry_type
+        self.rasterized: bool = code_interface.rasterized
 
         self.__data_to_update: bool = False
         self.__range_to_update: bool = False
@@ -98,7 +117,7 @@ class VisualizationPanel:
             Load input files
         """
 
-        def load_file(event, browser_name:str):
+        def load_file(event, browser_name: str):
             """Request the slave to load an input file. If the file is a geometry file, the slave is reseted
 
             Parameters
@@ -129,8 +148,6 @@ class VisualizationPanel:
         for name, description in file_input_list:
             self.file_browsers[name] = ServerFileBrowser(
                 name=name
-                # , sizing_mode="stretch_width"
-                #, description=description
             )
             self.file_browsers[name].param.watch(functools.partial(load_file, browser_name=name), "selected_file")
 
@@ -156,7 +173,7 @@ class VisualizationPanel:
         self.color_map_selector = pn.widgets.ColorMap(
             options=beautiful_color_maps,
             visible=False,
-            swatch_width = 60,
+            swatch_width=60,
         )
 
         self.color_map_selector.width = self.color_map_selector.height
@@ -167,13 +184,17 @@ class VisualizationPanel:
         self.color_map_selector.value_name = "BuRd"
         self.color_map_selector.value = beautiful_color_maps["BuRd"]
 
-        polygons, volume_compo_list, volume_color_list = self.compute_fn(
+        data_ = self.compute_fn(
             (1, 0, 0), (0, 1, 0), u_min, v_min, u_max, v_max, z
         )
 
-        self.plotter = Bokeh2DPlotter()
+        if self.display_polygons:
+            self.plotter = Bokeh2DPolygonPlotter()
+        else:
+            self.plotter = Bokeh2DGridPlotter()
+
         self.plotter.set_axes((1, 0, 0), (0, 1, 0), z)
-        self.plotter.plot_2d_frame(polygons, volume_compo_list, volume_color_list)
+        self.plotter.plot_2d_frame(data_)
 
         if (
             slave.get_label_coloring_mode(self.field_color_selector.value[0]) == VisualizationMode.FROM_VALUE
@@ -181,8 +202,8 @@ class VisualizationPanel:
             self.plotter.update_colorbar(
                 True,
                 (
-                    min([float(e) for e in volume_compo_list]),
-                    max([float(e) for e in volume_compo_list]),
+                    min([float(e) for e in data_.cell_values]),
+                    max([float(e) for e in data_.cell_values]),
                 ),
             )
         else:
@@ -202,7 +223,7 @@ class VisualizationPanel:
             figure=fig_pane,
             button_1=self.color_map_selector,
             button_2=hide_show_button,
-            message = pn.pane.Markdown(""),
+            message=pn.pane.Markdown(""),
             margin=0,
             width_policy="max",
             height_policy="max",
@@ -414,13 +435,13 @@ class VisualizationPanel:
         )
 
         self.axes_card = pn.Card(
-                pn.Column(axis_buttons, pn.Row(u, v)),
-                title="View axis coordinates",
-                width=350,
-                margin=(0, 0, 0, 0),
-                collapsed=True,
-            )
-        
+            pn.Column(axis_buttons, pn.Row(u, v)),
+            title="View axis coordinates",
+            width=350,
+            margin=(0, 0, 0, 0),
+            collapsed=True,
+        )
+
         file_loader_list = []
         for fi in self.file_browsers:
             file_loader_list.append(pn.pane.Markdown(f"{fi} file browser", margin=(0, 0, 0, 0)))
@@ -499,9 +520,22 @@ class VisualizationPanel:
             if pn.state.curdoc is not None:
                 pn.state.curdoc.add_next_tick_callback(self.async_update_data)
 
+        def field_changed(event):
+            """Function called on field changed
+
+            Parameters
+            ----------
+            event : Any
+                Field changed trigering event
+            """
+            if self.field_change_callback is not None and\
+                    len(self.field_color_selector.value) > 0:
+                self.field_change_callback(self.field_color_selector.value[0])
+            recompute_cb(event)
+
         # Attach the CB to the button
         self.recompute_btn.on_click(recompute_cb)
-        self.field_color_selector.param.watch(recompute_cb, "value")
+        self.field_color_selector.param.watch(field_changed, "value")
         self.color_map_selector.param.watch(recompute_cb, "value_name")
         self.center_colormap_on_zero_tick.param.watch(recompute_cb, "value")
 
@@ -556,26 +590,12 @@ class VisualizationPanel:
                 )
                 self.plotter.set_color_map(self.color_map_selector.value_name)
 
-            if "source_polygons" in self.__new_data:
-                self.current_polygons = self.__new_data["source_polygons"]
+            if "data" in self.__new_data:
+                self.current_data: Data2D = self.__new_data["data"]
                 if not self.update_polygons:
-                    new_compo_name = self.polygon_worker.sort_list(
-                        self.__new_data["source_polygons"][COMPO_NAMES]
-                    )
-                    new_color = self.polygon_worker.sort_list(
-                        self.__new_data["source_polygons"][COLORS]
-                    )
-
-                    self.plotter.update_colors(
-                        new_compo_name,
-                        new_color,
-                    )
+                    self.plotter.update_colors(self.current_data)
                 else:
-                    self.plotter.update_2d_frame(
-                        polygon_list=self.__new_data["source_polygons"][POLYGONS],
-                        compo_list=self.__new_data["source_polygons"][COMPO_NAMES],
-                        colors=self.__new_data["source_polygons"][COLORS],
-                    )
+                    self.plotter.update_2d_frame(self.current_data)
 
             self.__data_to_update = False
 
@@ -608,7 +628,7 @@ class VisualizationPanel:
                 self.x1_inp.value = self.__new_data["x1"]
             if "y1" in self.__new_data:
                 self.y1_inp.value = self.__new_data["y1"]
-                
+
             if "w" in self.__new_data:
                 self.w_inp.value = self.__new_data["w"]
 
@@ -621,21 +641,21 @@ class VisualizationPanel:
             pn.io.push_notebook(self.fig_overlay)
 
         if "field_name" in self.__new_data:
-            self.marked_to_recompute=False
+            self.marked_to_recompute = False
             if self.__new_data["field_name"] in self.field_color_selector.options:
-                self.field_color_selector.value = [self.__new_data["field_name"]]
+                if set(self.field_color_selector.value) != set([self.__new_data["field_name"]]):
+                    self.field_color_selector.value = [self.__new_data["field_name"]]
 
-                self.async_update_data()
+                    self.async_update_data()
         else:
             # If marked to recompute, a safe change was applied on a plot parameter, a recompute is requested async
             if self.marked_to_recompute:
                 self.recompute()
-                self.marked_to_recompute=False
+                self.marked_to_recompute = False
                 self.async_update_data()
 
         self.__new_data = {}
 
-
     def compute_fn(
         self,
         u: Tuple[float, float, float],
@@ -647,13 +667,8 @@ class VisualizationPanel:
         z: float,
         res_x: int = 300,
         res_y: int = 300,
-    ) -> Tuple[
-        List[PolygonElement],
-        List[Union[str, int]],
-        List[str],
-        List[Tuple[int, int, int]],
-    ]:
-        """Request the slave to compute a new frame, and updates the polygons to display
+    ) -> Data2D:
+        """Request the slave to compute a new frame, and updates the data to display
 
         Parameters
         ----------
@@ -678,8 +693,8 @@ class VisualizationPanel:
 
         Returns
         -------
-        List[PolygonElement], List[Union[str, int]], List[str], List[Tuple[int, int, int]]
-            Formatted list of polygons, list of volumes, list of compositions, list of volume colors.
+        Data2D
+            Geometry data.
         """
         if self.bounds_row is None:
             options = {
@@ -713,24 +728,26 @@ class VisualizationPanel:
                 print(
                     f"\n\n Got None from computed data on {self.name}, returning the past values.\n\n"
                 )
-                return (
-                    self.current_polygons[POLYGONS],
-                    self.current_polygons[COMPO_NAMES],
-                    self.current_polygons[COLORS],
-                )
-            polygon_list, dict_compos_found, dict_volume_color, polygons_updated = (
+                return self.current_data
+
+            computed_data, polygons_updated = (
                 computed_data
             )
-            self.update_polygons = polygons_updated
 
-            return self.polygon_worker.sort_polygon_list(
-                polygon_list,
-                dict_compos_found,
-                dict_volume_color,
-                sort=polygons_updated,
-            )
+            if polygons_updated or (self.polygon_sorter.sort_indexes is None):
+                self.polygon_sorter.sort_from_value(
+                    computed_data
+                )
+                self.update_polygons = True
+            else:
+                self.polygon_sorter.sort_list(
+                    computed_data
+                )
+                self.update_polygons = False
+
+            return computed_data
         else:
-            return None, None, None
+            return None
 
     def get_uv(self,) -> Tuple[np.ndarray, np.ndarray]:
         """Gets the normal direction vectors from the FloatInput objects.
@@ -768,7 +785,7 @@ class VisualizationPanel:
         if len(self.field_color_selector.value) == 0:
             # We can't process the calculation if no field is selected
             return
-        
+
         self.recompute_btn.disabled = True
         if profile_time:
             st = time.time()
@@ -793,22 +810,17 @@ class VisualizationPanel:
 
         print(f"{self.name} - Recomputing for axes {u}, {v}, at range : ({x0}, {y0}), ({x1}, {y1}), ({self.w_inp.value}), with field {self.field_color_selector.value[0]}")
 
-        polygons, volume_compo_list, volume_color_list = self.compute_fn(
+        data = self.compute_fn(
             u, v, x0, y0, x1, y1, self.w_inp.value, res_x=res_x, res_y=res_y
         )
 
-        if polygons is not None:
+        if data is not None:
             if profile_time:
                 print(f"Plot panel compute function : {time.time() - st}")
                 st = time.time()
 
             self.__new_data = {
-                "source_polygons": {
-                    POLYGONS: polygons,
-                    COLORS: volume_color_list,
-                    VOLUME_NAMES: [p.volume_id for p in polygons],
-                    COMPO_NAMES: volume_compo_list,
-                },
+                "data": data,
             }
 
             if (
@@ -816,8 +828,8 @@ class VisualizationPanel:
                 and self.slave.get_label_coloring_mode(self.field_color_selector.value[0]) == VisualizationMode.FROM_VALUE
             ):
                 self.__new_data["color_mapper"] = {
-                    "new_low": np.nanmin(np.array(volume_compo_list).astype(float)),
-                    "new_high": np.nanmax(np.array(volume_compo_list).astype(float)),
+                    "new_low": np.nanmin(np.array(data.cell_values).astype(float)),
+                    "new_high": np.nanmax(np.array(data.cell_values).astype(float)),
                 }
                 self.__new_data["hide_colorbar"] = False
             else:
@@ -859,8 +871,6 @@ class VisualizationPanel:
         new_visualiser = VisualizationPanel(self.slave, new_name)
         new_visualiser.copy_index = new_index
 
-        new_visualiser.plotter.source_polygons.data = self.plotter.source_polygons.data.copy()
-
         return new_visualiser
 
     def __get_option_widget(self, option: OptionElement) -> pn.widgets.Widget:
@@ -938,8 +948,8 @@ class VisualizationPanel:
         """
         return self.slave
 
-    def provide_on_mouse_move_callback(self, callback:Callable):
-        """Stores a function to call everytime the user moves the mouse on the plot. 
+    def provide_on_mouse_move_callback(self, callback: Callable):
+        """Stores a function to call everytime the user moves the mouse on the plot.
         Functions arguments are location, volume_id.
 
         Parameters
@@ -949,8 +959,8 @@ class VisualizationPanel:
         """
         self.plotter.provide_on_mouse_move_callback(callback)
 
-    def provide_on_clic_callback(self, callback:Callable):
-        """Stores a function to call everytime the user clics on the plot. 
+    def provide_on_clic_callback(self, callback: Callable):
+        """Stores a function to call everytime the user clics on the plot.
         Functions arguments are location, volume_id.
 
         Parameters
@@ -960,8 +970,18 @@ class VisualizationPanel:
         """
         self.plotter.provide_on_clic_callback(callback)
 
-        
-    def recompute_at(self, position:Tuple[float, float, float], volume_id:str):
+    def provide_field_change_callback(self, callback: Callable):
+        """Stores a function to call everytime the displayed field is changed.
+        the functions takes a string as argument.
+
+        Parameters
+        ----------
+        callback : Callable
+            Function to call.
+        """
+        self.field_change_callback = callback
+
+    def recompute_at(self, position: Tuple[float, float, float], volume_id: str):
         """Triggers a panel recomputation at the provided location. Called by layout update event.
 
         Parameters
@@ -985,14 +1005,16 @@ class VisualizationPanel:
             self.marked_to_recompute = True
             pn.state.curdoc.add_next_tick_callback(self.async_update_data)
 
-    def set_coordinates(self, 
-                            u:Tuple[float, float, float] = None,
-                            v:Tuple[float, float, float] = None,
-                            u_min:float = None,
-                            u_max:float = None,
-                            v_min:float = None,
-                            v_max:float = None,
-                            w:float = None,):
+    def set_coordinates(
+            self,
+            u: Tuple[float, float, float] = None,
+            v: Tuple[float, float, float] = None,
+            u_min: float = None,
+            u_max: float = None,
+            v_min: float = None,
+            v_max: float = None,
+            w: float = None,
+    ):
         """Updates the plot coordinates
 
         Parameters
@@ -1014,7 +1036,7 @@ class VisualizationPanel:
         """
         self.__data_to_update = True
         self.__range_to_update = True
-        
+
         if u is not None:
             if not type(u) in [tuple, list, np.ndarray]:
                 raise TypeError(f"u must have one of the following types: [tuple, list, np.ndarray], found {type(u)}")
@@ -1032,7 +1054,7 @@ class VisualizationPanel:
             self.__new_data["v0"] = v[0]
             self.__new_data["v1"] = v[1]
             self.__new_data["v2"] = v[2]
-            
+
         if u_min is not None:
             if not type(u_min) in [float, int]:
                 raise TypeError(f"u_min must be a number, found type {type(u_min)}")
@@ -1049,7 +1071,7 @@ class VisualizationPanel:
             if not type(v_max) in [float, int]:
                 raise TypeError(f"v_max must be a number, found type {type(v_max)}")
             self.__new_data["y1"] = v_max
-            
+
         if w is not None:
             if not type(w) in [float, int]:
                 raise TypeError(f"w must be a number, found type {type(w)}")
@@ -1059,17 +1081,26 @@ class VisualizationPanel:
         if pn.state.curdoc is not None:
             pn.state.curdoc.add_next_tick_callback(self.async_update_data)
 
-    def set_field(self, field_name:str):
+    def set_field(self, field_name: str, allow_wrong_name: bool = False):
         """Updates the plotted field
 
         Parameters
         ----------
         field_name : str
             New field to display
+        allow_wrong_name : bool
+            Accept a wrong field (nothing happens)
         """
-        if not field_name in self.field_color_selector.options:
+        if field_name not in self.field_color_selector.options:
+            if allow_wrong_name:
+                return
             raise ValueError(f"Requested field {field_name} not found, available fields : {self.field_color_selector.options}")
-        
-        self.__new_data["field_name"] = field_name
-        if pn.state.curdoc is not None:
-            pn.state.curdoc.add_next_tick_callback(self.async_update_data)
\ No newline at end of file
+
+        if set([field_name]) != set(self.field_color_selector.value):
+            self.__new_data["field_name"] = field_name
+
+            # Reseting indexes to prevent weird edges
+            self.polygon_sorter.reset_indexes()
+
+            if pn.state.curdoc is not None:
+                pn.state.curdoc.add_next_tick_callback(self.async_update_data)
