@@ -6,11 +6,18 @@ import time
 import pandas as pd
 from typing import Any, List, Dict, Tuple, Type, Union
 
+from scivianna.constants import OUTSIDE
+from scivianna.data import Data2D
 from scivianna.utils.color_tools import interpolate_cmap_at_values
 
-from scivianna.interface.generic_interface import GenericInterface, Geometry2D, IcocoInterface, ValueAtLocation, Value1DAtLocation
+from scivianna.interface.generic_interface import (
+    GenericInterface,
+    Geometry2D, 
+    IcocoInterface, 
+    ValueAtLocation, 
+    Value1DAtLocation
+)
 from scivianna.interface.option_element import OptionElement
-from scivianna.utils.polygonize_tools import PolygonElement
 from scivianna.enums import VisualizationMode
 
 from typing import TYPE_CHECKING
@@ -55,20 +62,20 @@ class SlaveCommand:
     """Returns the values of a field at cells"""
 
 
-def get_colors_list(
-    polygon_list: List[PolygonElement],
+def set_colors_list(
+    data: Data2D,
     code_interface: GenericInterface,
     coloring_label: str,
     color_map: str,
     center_colormap_on_zero: bool,
     options: Dict[str, Any],
-) -> Tuple[np.ndarray, Dict[int | str, str], Dict[int | str, Tuple[int, int, int]]]:
-    """Returns the list of color for a fiel per polygon.
+):
+    """Sets in a Data2D the list of colors for a field per polygon.
 
     Parameters
     ----------
-    polygon_list : List[PolygonElement]
-        List of polygon elements
+    data : Data2D
+        Geometry data
     code_interface : GenericInterface
         Code interface to request the field values
     coloring_label : str
@@ -80,11 +87,6 @@ def get_colors_list(
     options : Dict[str, Any]
         Plot extra options
 
-    Returns
-    -------
-    Tuple[np.ndarray, Dict[int | str, str], Dict[int | str, Tuple[int, int, int]]]
-        List of volume, list of materials, volume - color map
-
     Raises
     ------
     NotImplementedError
@@ -96,15 +98,14 @@ def get_colors_list(
     if not isinstance(code_interface, Geometry2D):
         raise TypeError("get_color_list can only be called with a Geometry2D code interface.")
         
-    # We list the volumes found, to assign a color per volume
-    list_volume_found = np.unique([p.volume_id for p in polygon_list])
-
     coloring_mode = code_interface.get_label_coloring_mode(coloring_label)
 
     dict_value_per_volume = code_interface.get_value_dict(
-        coloring_label, list_volume_found, options
+        coloring_label, data.cell_ids, options
     )
-
+    
+    cell_values = [dict_value_per_volume[v] for v in data.cell_ids]
+    
     if profile_time:
         print(f"get color list prepare time {time.time() - start_time}")
         start_time = time.time()
@@ -116,22 +117,25 @@ def get_colors_list(
         sorted_values = np.sort(np.unique(list(dict_value_per_volume.values())))
         map_to = np.array([hash(c)%255 for c in sorted_values]) / 255
 
-        compo_list = np.array([dict_value_per_volume[v] for v in list_volume_found])
+        value_list = np.array(cell_values)
 
-        _, inv = np.unique(compo_list, return_inverse=True)
+        _, inv = np.unique(value_list, return_inverse=True)
 
-        dict_volume_color = interpolate_cmap_at_values(
-            color_map, map_to[inv].astype(float), list_volume_found
+        volume_colors = interpolate_cmap_at_values(
+            color_map, map_to[inv].astype(float)
         )
-
-        dict_volume_color["-1"] = [255, 255, 255, 255]
+        
+        if OUTSIDE in data.cell_ids:
+            for index_ in np.where(data.cell_ids == OUTSIDE):
+                volume_colors[index_] = (255, 255, 255, 0)
 
     elif coloring_mode == VisualizationMode.FROM_VALUE:
         """
         The color is got from a color map set in the range (-max, max)
         """
-        dict_values = np.array([float(e) for e in dict_value_per_volume.values()])
-        no_nan_values = dict_values[~np.isnan(dict_values)]
+        cell_values = np.array(cell_values).astype(float)
+        no_nan_values = cell_values[~np.isnan(cell_values)]
+
         if profile_time:
             print(f"extracting no nan {time.time() - start_time}")
             start_time = time.time()
@@ -144,7 +148,7 @@ def get_colors_list(
             else:
                 minmax = max(abs(no_nan_values.min()), no_nan_values.max())
 
-            dict_values = (dict_values + minmax) / (2 * minmax)
+            cell_values = (cell_values + minmax) / (2 * minmax)
         else:
             if (
                 len(no_nan_values) == 0 or max(abs(no_nan_values.min()), no_nan_values.max()) == 0.0
@@ -158,13 +162,14 @@ def get_colors_list(
                 minmax = no_nan_values.max() - no_nan_values.min()
                 min_val = no_nan_values.min()
 
-            dict_values = (dict_values - min_val) / minmax
+            cell_values = (cell_values - min_val) / minmax
 
         if profile_time:
             print(f"Rescaling data {time.time() - start_time}")
             start_time = time.time()
-        dict_volume_color = interpolate_cmap_at_values(
-            color_map, dict_values, list_volume_found
+
+        volume_colors = interpolate_cmap_at_values(
+            color_map, cell_values
         )
 
         if profile_time:
@@ -172,9 +177,9 @@ def get_colors_list(
             start_time = time.time()
 
         # Changing the main color from black to gray in case of Nan
-        for c in dict_volume_color:
-            if dict_volume_color[c][3] == 0.0:
-                dict_volume_color[c] = (200, 200, 200, 0)
+        for c in range(len(volume_colors)):
+            if volume_colors[c, 3] == 0.0:
+                volume_colors[c] = (200, 200, 200, 0)
 
         if profile_time:
             print(f"Fixing nans {time.time() - start_time}")
@@ -184,18 +189,14 @@ def get_colors_list(
         """
         No color, mesh displayed only
         """
-        dict_volume_color = dict(
-            zip(
-                list(list_volume_found) + ["-1"],
-                [(200, 200, 200, 0)] * (len(list_volume_found) + 1),
-            )
-        )
+        volume_colors = np.array([(200, 200, 200, 0)] * (len(data.cell_ids)))
     else:
         raise NotImplementedError(
             f"Visualization mode {coloring_mode} not implemented."
         )
-
-    return list_volume_found, dict_value_per_volume, dict_volume_color
+    
+    data.cell_values = cell_values
+    data.cell_colors = volume_colors.tolist()
 
 
 def worker(
@@ -267,7 +268,7 @@ def worker(
                     raise TypeError(
                         f"The requested panel is not associated to an Geometry2D, found class {type(code_)}."
                     )
-                polygon_list, polygons_updated = code_.compute_2D_data(
+                data, polygons_updated = code_.compute_2D_data(
                     u,
                     v,
                     u_min,
@@ -285,15 +286,13 @@ def worker(
                     print(f"Code compute 2D time : {time.time() - st}")
                     st = time.time()
 
-                list_volume_found, dict_compos_found, dict_volume_color = (
-                    get_colors_list(
-                        polygon_list,
-                        code_,
-                        coloring_label,
-                        color_map,
-                        center_colormap_on_zero,
-                        options,
-                    )
+                set_colors_list(
+                    data,
+                    code_,
+                    coloring_label,
+                    color_map,
+                    center_colormap_on_zero,
+                    options,
                 )
 
                 if profile_time:
@@ -302,9 +301,7 @@ def worker(
 
                 q_returns.put(
                     [
-                        polygon_list,
-                        dict_compos_found,
-                        dict_volume_color,
+                        data,
                         polygons_updated,
                     ]
                 )
@@ -523,9 +520,7 @@ class ComputeSlave:
         center_colormap_on_zero: bool,
         options: Dict[str, Any],
     ) -> Tuple[
-        List[PolygonElement],
-        Dict[int | str, str],
-        Dict[int | str, Tuple[int, int, int]],
+        Data2D, bool
     ]:
         """Get the geometry from the interface
 
@@ -560,8 +555,8 @@ class ComputeSlave:
 
         Returns
         -------
-        Tuple[List[PolygonElement], Dict[int | str, str], Dict[int | str, Tuple[int, int, int]]]
-            List of PolygonElements, list of materials, volume - color map
+        Tuple[Data2D, bool]
+            Data2D object containing the geometry, whether the polygons were updated
         """
         self.q_tasks.put(
             [
